#include <stdlib.h>
#include <algorithm> 

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
int Read_node_id(char *file_name)
// added by Lubos
// read node id for nodes which should be included in the calculation
{
	FILE *data_file;
	if((data_file = fopen(file_name,"r"))==NULL) return 0;

	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	if (verbose_jlm)
	{
		fprintf (stderr,"Reading node ids ...\n ");
	}
	try{
		fscanf(data_file,"%i\n",&nodes);
		node_id.resize(nodes);										
		for(int i=0;i<nodes;i++)
		{
			fscanf(data_file,"%i\n",&node_id[i]);		
			}
		fclose (data_file);
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while reading node_id." << endl;
		return 0;
	}
	return 1;

}
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
int Read_poc_obyv(char *file_name)
// added by Lubos
// read node id for nodes which should be included in the calculation
{
	FILE *data_file;
	if((data_file = fopen(file_name,"r"))==NULL) return 0;

	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	if (verbose_jlm)
	{
		fprintf (stderr,"Reading poc_obyv ...\n ");
	}
	try{
		fscanf(data_file,"%i\n",&all_nodes);
		poc_obyv.resize(all_nodes);										
		for(int i=0;i<all_nodes;i++)
		{
			fscanf(data_file,"%i\n",&poc_obyv[i]);		
		}
		fclose(data_file);
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while reading poc_obyv." << endl;
		return 0;
	}
	return 1;
}
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
int Read_all_distances(char *file_name)
// added by Lubos
// read node id for nodes which should be included in the calculation
{
	FILE *data_file;
	if((data_file = fopen(file_name,"r"))==NULL) return 0;

	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	if (verbose_jlm)
	{
		fprintf(stderr,"Reading all distances  ...\n");	
	}
	try
	{
		fscanf(data_file,"%i\n",&all_nodes);
		fscanf(data_file,"%i\n",&all_nodes);
		cost.resize(all_nodes);												/////////////// MEMORY ///////////////
			for(int i=0;i<all_nodes;i++)
			{
				cost[i].resize(all_nodes);									/////////////// MEMORY ///////////////
				for(int j=0;j<all_nodes;j++)
				{
					fscanf(data_file,"%i\n",&cost[i][j]);	
				}
			}
		fclose (data_file);
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while reading all distances" << endl;
		return 0;
	}
	return 1;
}
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
int Read_p_values(char *file_name)
// added by Lubos
// read node id for nodes which should be included in the calculation
{
	FILE *data_file;
	if((data_file = fopen(file_name,"r"))==NULL) return 0;

	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	if (verbose_jlm)
	{
		fprintf (stderr,"Reading p values ...\n ");
	}
	try{
		fscanf(data_file,"%i\n",&num_p_values);
		p_values.resize(num_p_values);										
		for(int i=0;i<num_p_values;i++)
		{
			fscanf(data_file,"%i\n",&p_values[i]);		
		}
		fclose (data_file);
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while reading p_values." << endl;
		return 0;
	}
	return 1;

}
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
bool Read_Parameters(int argc,char *argv[])
{
	if(argc<2) return false;
	char *file_name;
	file_name = argv[1];
	if (strlen(file_name)>=5)
	{
		if(strcmp(&file_name[strlen(file_name)-4], ".tsp") == 0) data_format=1;
		else if(strcmp(&file_name[strlen(file_name)-7], ".dimacs") == 0)
		{
			data_format=2;
		}
		else
		{
			if (verbose_zebra)
			{
				printf("ERROR: data extension not recognized.\n");
				printf("Valid extensions: .tsp, .dimacs\n");
			}
			return false;
		}
	}
	int arg = 2;
	while(arg<argc)
	{
		if(strcmp(arg[argv],"-p")==0)
		{
			++arg;
			if(arg==argc) return false;
			p = atoi(argv[arg]);
			if(p<=0)
			{
				if (verbose_zebra)
				{
					printf("ERROR: p must be strictly positive.\n");
				}
				return false;
			}
			no_p = false;
			++arg;
			continue;
		}

		if(strcmp(arg[argv],"-b")==0)
		{
			++arg;
			if(arg==argc) return false;
			incumbent_value = atof(argv[arg]);
			++arg;
			continue;
		}

		if(strcmp(arg[argv],"-h")==0)
		{
			RW = true;
			++arg;
			if(arg==argc) return false;
			RWiter = atoi(argv[arg]);
			++arg;
			continue;
		}

		if(strcmp(arg[argv],"-s")==0)
		{
			save_solution_info = true;
			++arg;
			if(arg==argc) return false;
			solution_file_name= argv[arg];
			++arg;
			continue;
		}

		if(strcmp(arg[argv],"-d")==0)
		{
			++arg;
			if(arg==argc) return false;
			int d = atoi(argv[arg]);
			if(d==1) allD = true;
			else if(d!=0) return false;
			++arg;
			continue;
		}

		return false;
	}

	if(no_p)
	{
		if (verbose_zebra)
		{
			printf("ERROR: No value for p has been given.\n");
		}
		return false;
	}
	return true;
}

//int ctsp(int i,int j)
//{
//	double a = x[i] - x[j];
//	double b = y[i] - y[j];
//	a *= a;
//	b *= b;
//	return (int)sqrt(a+b);	
//}


//int cdimacs(int i,int j)
//{
//	if(i<j) return cost[i][j-i-1];
//	if(i>j) return cost[j][i-j-1];
//	return 0;
//}

int D(int i,int k, int (*Cij)(int ii, int jj))
{
	if (k<Dv[i].size()) return Dv[i][k];
	else
	{
		int aux,aux2;
		int bestval = intM;
		aux2 = Dv[i][k-1];
		for(int h=0;h<nodes;h++)
		{
			aux = Cij(h,i);		// customer i
			if( (aux<bestval-tolerance) && (aux>aux2+tolerance) )
			{
				bestval = aux;
				if(aux==aux2+1) break;
			}
		}
		Dv[i].push_back(bestval);									///////////////MEMORY////////////////////
		return bestval;
	}
}

/* Read the initial data: nodes, p, cost[i][j] */
int Read_Data(char *file_name)
{
	FILE *data_file;
	if((data_file = fopen(file_name,"r"))==NULL) return 0;
	
	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	if (data_format==1)
	{
		if (verbose_zebra)
		{
			fprintf (stderr,"Reading TSP instance... ");
		}		
		double vx, vy;
		int v;
		bool size_read = false;
		while (fgets (buffer, LINESIZE, data_file))
		{
			if (sscanf(buffer, "DIMENSION : %d", &nodes) == 1) 
			{
				size_read = true;
				x = new double[nodes]; 							////////////////// MEMORY //////////////////
				y = new double[nodes];							////////////////// MEMORY //////////////////
				continue;
			}
			if (sscanf(buffer, "%d %lg %lg", &v, &vx, &vy) == 3)
			{
				if (!size_read) 
				{
					if (verbose_zebra)
					{
						printf("ERROR: wrong data structure in file %s\n",file_name);
					}
					return 0;
				}
				x[v-1] = vx;
				y[v-1] = vy;
				continue;
			}
		}
	}
	else
	{
		if (verbose_zebra)
		{
			fprintf (stderr,"Reading DIMACS instance... ");
		}
		fscanf(data_file,"%i",&nodes);
		int pp;
		fscanf(data_file,"%i",&pp);
		
		try{
			cost.resize(nodes-1);										/////////////// MEMORY ///////////////
			for(int i=0;i<nodes-1;i++)
			{
				cost[i].resize(nodes-1-i);								/////////////// MEMORY ///////////////
				for(int j=i+1;j<nodes;j++)
				{
					fscanf(data_file,"%i",&cost[i][j-i-1]);				       		//////////////  MEMORY ////////////////
				}
			}
			while (fgets (buffer, LINESIZE, data_file))
			{
				if (sscanf(buffer, "Create data time: %lf", &external_generation_time) == 1)
				{
					break;
				}
			}
		}
		catch(const bad_alloc)
		{
			cout << "ERROR: out of memory while reading costs." << endl;
			return 0;
		}
	}
	if (verbose_zebra)
	{
		fprintf (stderr, "done.\n");
	}
	fclose(data_file);
	return 1;
}

bool my_order_criterion (int i,int j)
{
	return (i<j);
}

/* Sort an array v from the least value to the largest one */
/* Remove repeated elements */
/* Return since of this new without-multiplicties vector */
int Sort_Int_Array(vector<int> &v)
{
	/* Sort vector */
	sort(v.begin(),v.end(),my_order_criterion);	
	/* Remove p-1 largest costs */
	v.erase(v.end()-p+1,v.end());
	/* Remove repeated elements */
	vector<int>::iterator temp_iterator;
	temp_iterator = v.begin();
	while(temp_iterator!=(v.end()-1))
	{
		if (abs(*temp_iterator-*(temp_iterator+1))<tolerance)
		{
			v.erase(temp_iterator+1);
		}
		else ++temp_iterator;
	}
	v.push_back(intM);
	return v.size();
}

void Generate_D_sdznoh(int (*Cij)(int ii, int jj))
{
	// Generate D
	// Generate sdz without starting heuristic information
	G = new int[nodes];											//////////////////// MEMORY ///////////////////
	/* Sort costs increasingly and without multiplicties */
	Dv.resize(nodes);											//////////////////// MEMORY ///////////////////
	int min_G = nodes;
	vector<int> aux;
	double sdzf;
	if(allD)
	{
		for(int i=0;i<nodes;i++)
		{
			aux.resize(nodes);				///////////////////// MEMORY   OK ////////////////////////
			for(int j=0;j<nodes;j++) aux[j] = Cij(j,i);     // customer i
			// We substrct one because of the artificial largest cost
			G[i] = Sort_Int_Array(aux) - 1;
			if(G[i]<min_G) min_G = G[i];
			Dv[i] = aux;
			aux.clear();					///////////////////// MEMORY   OK ////////////////////////
		}
		int totalG; //totalG[i] = G[0] + ... + G[i]
		totalG = G[0];
		for(int i=1;i<nodes;i++) totalG+= G[i];
		sdzf = (totalG/nodes)/p;
	}
	else
	{

	
		for(int i=0;i<nodes;i++)
		{
			Dv[i].resize(1);							/////////////////// MEMORY OK //////////////////////////
			Dv[i][0] = 0;
			G[i] = nodes - p + 1; // Artificial upper bound. 
		}

		// Estimate sdz
		// Let u = ceil(alpha*nodes), 0 < alpha <= 1.
		// Select nodes floor(h*(nodes-1)/u), h=0,1,2,...,u.
		
		int partialG = 0;
		int samplesize = (int)ceil((double)alpha*nodes);
		int Gi,k;
		for(int h=0;h<=samplesize;h++)
		{
			k = (int)floor((double)h*(nodes-1)/samplesize);
			aux.resize(nodes);								/////////////////// MEMORY ////////////////////
			for(int j=0;j<nodes;j++) aux[j] = Cij(j,k);  //customer k
			// We substrct one because of the artificial largest cost
			Gi= Sort_Int_Array(aux) - 1;
			partialG+= Gi;
			if(Gi<min_G) min_G = Gi;
			aux.clear();									////////////////////////// MEMORY //////////////////////

		}
		sdzf = partialG/((1+samplesize)*p);
	}
	if(sdzf>min_G) sdzf = min_G;
	// Reduce sdzf if the initial number of constraints is large 
	double auxf = (sdzf-2)*nodes;
	if(auxf<10000) sdzf*= 2;
	else sdzf*= log10(1000.0)/log10(auxf);
	
	sdz = (int)ceil(sdzf);
	if(sdz<3) sdz=3;
	sdz-= 2;
	printf("SDZ: %i\n",sdz);
	ind_sdz.resize(nodes);									      //////////////////////////// MEMORY //////////////////////
	for(int i=0;i<nodes;i++) ind_sdz[i] = sdz;
	return;
}

void Generate_D_sdzh(int (*Cij)(int ii, int jj))
{
	// Generate D
	// Generate sdz with starting heuristic information
	
	/* Sort costs increasingly and without multiplicties */
	Dv.resize(nodes);									////////////////// MEMORY ////////////////
	vector<int> aux;
	if(allD)
	{
		for(int i=0;i<nodes;i++)
		{
			aux.resize(nodes);						////////////////// MEMORY ////////////////
			for(int j=0;j<nodes;j++) aux[j] = Cij(j,i);  // customer i
			Sort_Int_Array(aux);
			Dv[i] = aux;
			aux.clear();							////////////////// MEMORY ////////////////
		}
	}
	else
	{
		for(int i=0;i<nodes;i++)
		{
			Dv[i].resize(1);							////////////////// MEMORY ////////////////
			Dv[i][0] = 0;
		}
	}
	
	// Get depths from the heuristic solution
	double refv;
	int k;
	ind_sdz.resize(nodes);									////////////////// MEMORY ////////////////
	int totz;
	totz = 0;
	for(int i=0;i<nodes;i++)
	{
		refv = Cij(i,incumbent_x[i]);
		k = 1;
		while(D(i,k,Cij)<refv+tolerance) ++k;
		ind_sdz[i] = k;
		totz+= ind_sdz[i];
	}
	sdz = (int)ceil((double)totz/nodes);
	if(sdz<2) sdz=2;
	//sdz-= 2;
	sdz= (int)(1.1*sdz);
	for(int i=0;i<nodes;i++)  ind_sdz[i] = sdz;
	return;
}

void Start_S()
{
	S.resize(nodes);										////////////////// MEMORY ////////////////
	for(int i=0;i<nodes;i++)
	{
		S[i].resize(1);										////////////////// MEMORY ////////////////
		S[i][0].push_back(i);
	}
	return;
}

void myprintf(char *v,int i1,int i2,int i3)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v,i1,i2,i3);
		else printf(v,i1,i2,i3);
	}
	return;
}

void myprintf(char *v)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v);
		else printf(v);
	}
	return;
}

void myprintf(char *v1, char *v2)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v1,v2);
		else printf(v1,v2);
	}
	return;
}

void myprintf(char *v, double d)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v,d);
		else printf(v,d);
	}
	return;
}

void myprintf(char *v, int i)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v,i);
		else printf(v,i);
	}
	return;
}

void myprintf(char *v,int i1, int i2)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v,i1,i2);
		else printf(v,i1,i2);
	}
	return; 
}

void myprintf(char *v,double i1, double i2)
{
	if (verbose_zebra)
	{
		if(save_solution_info) fprintf(solution_file,v,i1,i2);
		else printf(v,i1,i2);
	}
	return;
}

void Show_Partial_Solution(void)
{
	myprintf("Total running time: %.0lf\n",(double)(clock()- start_time)/CLOCKS_PER_SEC);
	myprintf("Best lower bound: %lf\n",bestLB);
	if(incumbent_value<IloInfinity)
	{
		myprintf("Best upper bound: %lf\n",incumbent_value);
		if(bestLB>tolerance) myprintf("Gap: %.4lf%%\n",100*(incumbent_value/bestLB-1));
	}
	else myprintf("Best upper bound: none\n");
	myprintf("SDZ: %i\n",sdz);
	return;
}

void Generate_S(int i,int k,int (*Cij)(int ii, int jj))
{
	/*Generate S[i][k] */
	try{
		S[i].resize(k+1);							////////////////// MEMORY ////////////////
		int val = D(i,k,Cij);
		for(int j=0;j<nodes;j++)
		{
			if(abs(Cij(j,i)-val)<tolerance) S[i][k].push_back(j); // customer i
		}
		return;
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while constructing S." << endl;
		Show_Partial_Solution();
		exit(1);
	}
}

i_l *Create_i_l(void)
{
	try
	{
		i_l * temp = new i_l;
		temp->next = NULL;	
		return temp;
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while adding new rows." << endl;
		Show_Partial_Solution();
		exit(1);
	}
}

n_i *Create_n_i(void)
{
	try	
	{
		n_i * temp		= new n_i;
		temp->parent_node 	= NULL;
		temp->left_son          = NULL;
		temp->right_son         = NULL; 
		return temp;
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while generating node info." << endl;
		Show_Partial_Solution();
		exit(1);
	}
}

a_n *Create_a_n(void)
{
	try
	{
		a_n * temp		= new a_n;
		temp->next 		= NULL;
		temp->previous		= NULL;
		temp->the_info		= NULL;
		return temp;
	}
	catch(const bad_alloc)
	{
		cout << "ERROR: out of memory while generating node info." << endl;
		Show_Partial_Solution();
		exit(1);
	}
}

