/* Get_Basis_Name(i).c_str() is the name of the file where the basis is saved. 
   It must be of the type const char* */
void Get_Basis_Name(int i, char* aux)
{
	sprintf(aux,"%d.bas",i);
}

void Get_y_Name(int i,char* aux)
{
	sprintf(aux,"y%d",i);
}

void Get_z_Name(int i,int j,char* aux)
{
	sprintf(aux,"z%d,%d",i,j);
}

void Get_c_Name(int i,int j,char* aux)
{
	sprintf(aux,"c%d.%d",i,j);
}

bool Check_Integer_y(IloNumArray y_value)
{
	int i=0;
	while(i<nodes)
	{
		if ( (y_value[i]>tolerance) && (y_value[i]<1-tolerance) ) return false;
		++i;
	}	
	return true;
}

double Score(double v1, double v2, int mod)
{
	if(mod==1)
	{
		double w1 = v1;
		double w2 = v2;
		if(v1<epsilon) w1 = epsilon;
		if(v2<epsilon) w2 = epsilon;
		return w1*w2;	
	}
	else
	{
		if(v1>v2) return (mu*v1 + (1-mu)*v2);
		return ((1-mu)*v1 + mu*v2);
	}
}

void UpdateAvgPs(void)
{
	double tl,tr;
	tl = tr = 0;
	vector<int>::iterator v1,v2;
	v1 = initpsl.begin();
	v2 = initpsl.end();
	while(v1!=v2)
	{
		tl+= totaldeltal[*v1]/nodesl[*v1];
		tr+= totaldeltar[*v1]/nodesr[*v1];
		++v1;
	}		
	avgpsl = tl/initpsl.size();
	avgpsr = tr/initpsr.size();
	return;
}

double PseudocostL(int i)
{
	if(nodesl[i]==0)
	{
		// Uninitialized left pseudocost
		if(initpsl.size()==0)
		{
			// No initialized left pseudocost
			return 1;
		}
		else return avgpsl;
	}
	else
	{
		// Initialized left pseudocost
		return totaldeltal[i]/nodesl[i];
	}
}

double PseudocostR(int i)
{
	if(nodesr[i]==0)
	{
		// Uninitialized right pseudocost
		if(initpsr.size()==0)
		{
			// No initialized right pseudocost
			return 1;
		}
		else return avgpsr;
	}
	else
	{
		// Initialized left pseudocost
		return totaldeltar[i]/nodesr[i];
	}
}

bool compsc(s_l e1, s_l e2)
{
	return(e1.sc>e2.sc);
}

int ReliabilityChooseBranchingy(IloNumVarArray &y, IloRangeArray &zconst, IloCplex &cplex,double objparent, bool yes_z_rows)
{
	// Get scores
	IloNumArray yv(y.getEnv(),nodes);
	cplex.getValues(yv,y);
	vector<s_l> candlist;	
	s_l auxs;
	double auxv;
	int i;
	int ScoreMode = 1; // 1 = Product, 2 = Mu
	for(i=0;i<nodes;i++)
	{
		auxv = yv[i];
		if( (auxv>tolerance) && (auxv<1-tolerance) )
		{
			auxs.i = i;
			auxs.sc = Score(yv[i]*PseudocostL(i),(1-yv[i])*PseudocostR(i),1);
			candlist.push_back(auxs);						/////////////////////// MEMORY //////////////////////
		}
	}
	sort(candlist.begin(),candlist.end(),compsc);
	vector<s_l>::iterator ss;
	ss = candlist.end()-1;
	if(ss->sc<1+tolerance)
	{
		// There is no pseudocost larger than 1.
		// Change to mu criterion instead of product.
		ScoreMode = 2;
		ss = candlist.begin();
		while(ss!=candlist.end())
		{
			i = ss->i;
			ss->sc = Score(yv[i]*PseudocostL(i),(1-yv[i])*PseudocostR(i),2);
			++ss;
		}
		sort(candlist.begin(),candlist.end(),compsc);
	}
	
	/* Choose the best among the reliable candidates */
	ss = candlist.begin();
	double best_val = -1;
	int best_i = -1;
	while(ss!=candlist.end())
	{
		i = ss->i;
		if ( (nodesl[i]>=reliab) && (nodesr[i]>=reliab) )
		{
			best_val = ss->sc;
			best_i = i;
			break;
		}
		++ss;
	}

	// Strong branching for some unreliable candidates
	int noimproveiter = 0;
	double auxvs,vall,valr;
	char bsname[15];
	IloCplex::BasisStatusArray yst(y.getEnv());
	IloCplex::BasisStatusArray rngst(y.getEnv());
	clock_t ttime;
	ss = candlist.begin();
	bool stronginitialized = false;
	while( (ss!=candlist.end()) && (noimproveiter<lambda) )
	{
		i = ss->i;
		if( (nodesl[i]<reliab) || (nodesr[i]<reliab) )
		{	
			totalstrong+=2;
			if(!stronginitialized)
			{
				// Set up information first time strong branching called
				stronginitialized = true;
				cplex.setParam(IloCplex::ItLim,75);
				
				
				if(yes_z_rows==false)
				{
					clock_t auxi_time=clock();
					++totalbasis;
					this_active_node->basis = totalbasis;
					Get_Basis_Name(totalbasis,bsname);
					cplex.writeBasis(bsname);
					write_basis_time+=(double)(clock()-auxi_time)/CLOCKS_PER_SEC;
				}
				else {
					 Get_Basis_Name(this_active_node->basis,bsname);
					 //printf("this_active_node->basis = %i\n",this_active_node->basis);
					 //printf("bsname = %s\n",bsname);

				}

				cplex.getBasisStatuses(yst,y,rngst,zconst);
			}
			// Strong branching downwards
			cplex.setBasisStatuses(yst,y,rngst,zconst);
			ttime = clock();
			//printf("1: ______________Going to read Basis_______________\n");
			//printf("bsname  = %s", bsname);
			cplex.readBasis(bsname);
			//printf("_______________Basis has been read_______________\n");
			strongb_readb_time+=(double)(clock()-ttime)/CLOCKS_PER_SEC;
			y[i].setBounds(0,0);
			ttime = clock();
			cplex.solve();
			strongb_solve_time+=(double)(clock()-ttime)/CLOCKS_PER_SEC;
			if(cplex.getStatus()!=IloAlgorithm::Infeasible)
			{
				vall = cplex.getObjValue() - objparent;
				totaldeltal[i]+= vall/yv[i];
				if(nodesl[i]==0) initpsl.push_back(i);					////////////// MEMORY //////////////////
				++nodesl[i];
			}
			else vall = -2;
						
			// Strong branching upwards
			cplex.setBasisStatuses(yst,y,rngst,zconst);
			ttime = clock();
			//printf("2: +++++++++++ going to read BASIS +++++++++++++++\n");
			cplex.readBasis(bsname);
			//printf("+++++++++++++  BASIS has been read  +++++++++++++++\n");


			strongb_readb_time+=(double)(clock()-ttime)/CLOCKS_PER_SEC;
			y[i].setBounds(1,1);
			ttime = clock();
			cplex.solve();
			strongb_solve_time+=(double)(clock()-ttime)/CLOCKS_PER_SEC;
			if(cplex.getStatus()!=IloAlgorithm::Infeasible)
			{
				valr = cplex.getObjValue() - objparent;
				totaldeltar[i]+= valr/(1-yv[i]);
				if(nodesr[i]==0) initpsr.push_back(i);					/////////////// MEMORY ////////////////////
				++nodesr[i];
			}
			else valr = -2;
					
			// Reset bound for y[i]
			y[i].setBounds(0,1);
			
			// Get strong branching score
			if( (vall>-tolerance) && (valr>-tolerance) )
			{
				// Both problems are feasible
				auxvs = Score(vall,valr,ScoreMode);
			}
			else if (vall>-tolerance)
			{
				// Right problem is unfeasible
				auxvs = Score(vall,vall,ScoreMode);
			}
			else
			{
				// Left problem is unfeasible
				auxvs = Score(valr,valr,ScoreMode);
			}

			if (auxvs>best_val+tolerance)
			{
				best_val = auxvs;
				best_i = i;
				if( (ss!=candlist.begin()) && (noimproveiter==lambda-1) )
				{
					// Increase lookahed step
					++lambda;
				}
				noimproveiter = 0;
			}
			else ++noimproveiter;
		}
		else ++noimproveiter;
		++ss;
	}
	
	if( (noimproveiter==lambda) && (lambda>lambdamin) )
	{		
		// Decrease lookahead step
		--lambda;
	}
	UpdateAvgPs();
	candlist.clear();	///////////////////// MEMORY ////////////		
	return best_i;
}

void Prune_Node(n_i *n_to_prune)
{
	if (n_to_prune != NULL) {
		n_i *parent_node = n_to_prune->parent_node;
		if (parent_node!=NULL)
		{
			//printf("parent != NULL\n");
			if (n_to_prune->branch_value==0)   // changed from =  to == by Lubos
				parent_node->left_son = NULL;
			else 
				parent_node->right_son = NULL;
			
			n_to_prune->parent_node = NULL;
			delete n_to_prune;
			--parent_node->active_children;
			if (parent_node->active_children==0) Prune_Node(parent_node);
		}
		else {
			//printf("parent == NULL\n");
			delete n_to_prune;
		}
	} else {
		//printf("n_to_prune == NULL\n");
	}
	return;
}

void Insert_in_List(n_i *n_to_insert)
{
	/* If the node cannot be pruned, it is added to the active node list */
	++active_nodes;
	a_n *aux = the_first_active_node;
	
	/* Best node search: lower bounds go first.
	   Should there be a tie, the newest node goes last. */
	while( (aux->next!=NULL) && (aux->the_info->lower_bound<n_to_insert->lower_bound+tolerance) )
	{
		aux = aux->next;
	}

	/* The first value has always a lesser (or equal) lower bound than the one in the node 
	   to insert. */
	if ( (aux->next==NULL) && (aux->the_info->lower_bound < n_to_insert->lower_bound+tolerance) )
	{

		aux->next = Create_a_n();
		aux->next->previous = aux;
		aux = aux->next;
		aux->next = NULL;
	}
	else
	{
		aux = aux->previous;
		a_n* aux2 = Create_a_n();
		aux2->previous = aux;
		aux2->next = aux->next;
		aux->next->previous = aux2;
		aux->next = aux2;
		aux = aux->next;
	}
	aux->the_info = n_to_insert;
	return;
}

void Set_Solver_Parameters(IloCplex cplex)
{
	cplex.setParam(IloCplex::RootAlg,2);
	cplex.setParam(IloCplex::PreInd,0); 
	cplex.setParam(IloCplex::DPriInd,2);
	cplex.setParam(IloCplex::APIEncoding,"UTF-8");
	cplex.setParam(IloCplex::FileEncoding,"UTF-8");

	return;
}

int Enlarge_zdepths(IloNumVarArray2 &zz)
{
	int zs = zdepths.size();							////////////// MEMORY //////////////
	zdepths.resize(zs+1);
	zdepths[zs].resize(nodes);
	for(int i=0;i<nodes;i++) zdepths[zs][i] = zz[i].getSize();
	return zs;
}

void Use_RW_info(char *filename)
{
	FILE *data_file;
	data_file = fopen(filename,"r");
	const int LINESIZE = 256;
	char buffer [LINESIZE + 1];
	double zvalue;
	int i,j,c,k;
	k=0;
	while(fgets (buffer, LINESIZE, data_file))
	{
		if(sscanf(buffer, "v %lf", &zvalue) == 1) 
		{
			incumbentis = true;
			if(zvalue<incumbent_value-tolerance) incumbent_value = zvalue;
			else return;
		}
		if(sscanf(buffer,"n %i",&nodes)==1) incumbent_x.resize(nodes);			////////////// MEMORY //////////////
		if(sscanf(buffer,"y %i", &j)==1)
		{
			incumbent_y[k] = j-1;
			++k;
		}
		if(sscanf(buffer,"x %d %d %d",&i,&j,&c)==3) incumbent_x[i-1] = j-1;
	}
	fclose (data_file);

}

void UpdatePseudocost(double parentobj, double childobj, bool kinship, double yval, int i)
{
	if(kinship)
	{
		// Right son
		totaldeltar[i]+= (childobj - parentobj)/(1-yval);
		if(nodesr[i]==0) initpsr.push_back(i);					////////////// MEMORY //////////////
		++nodesr[i];
	}
	else
	{
		// Left son
		totaldeltal[i]+= (childobj - parentobj)/yval;
		if(nodesl[i]==0) initpsl.push_back(i);					////////////// MEMORY //////////////
		++nodesl[i];
	}
}

void DeleteTree()
{
	if (the_first_active_node != NULL) {
		while(the_first_active_node->next!=NULL)
		{
			a_n *aux_an;
			aux_an = the_first_active_node;
			the_first_active_node = the_first_active_node->next;
			the_first_active_node->previous = NULL;
			aux_an->next = NULL;
			this_active_node = aux_an->the_info;
			aux_an->the_info = NULL;
			Prune_Node(this_active_node);
			delete aux_an;
		}
		this_active_node = the_first_active_node->the_info;
		the_first_active_node->the_info = NULL;
		Prune_Node(this_active_node);
		delete(the_first_active_node);
	}
	return;
}

void PrintSolInfo()
{
	myprintf("\n");
	myprintf("TOTAL NODES: %i\n",total_nodes);
	myprintf("LOWER BOUND AT ROOT: %i\n",(int)LB_root);
	myprintf("\n");
	myprintf("* CREATE DATA TIME: %.2lf\n",create_data_time);
	myprintf("* GENERATE MODEL TIME: %.2lf\n",generate_model_time);
	myprintf("* ADD CONSTRAINTS TIME: %.2lf\n",add_constraints_time);
	myprintf("* WRITE BASIS TIME: %.2lf\n",write_basis_time);
	myprintf("* READ BASIS TIME: %.2lf\n",read_basis_time);
	myprintf("* SOLVE MODEL TIME: %.2lf\n",solve_model_time);
	myprintf("\n");
	if(total_nodes>0)
	{
		myprintf("* CHOOSE BRANCHING Y: %.2lf\n",choose_y);
		myprintf(" STRONG BRANCHING - READ BASIS: %.2lf\n",strongb_readb_time);
		myprintf(" STRONG BRANCHING - SOLVE: %.2lf\n",strongb_solve_time);
		myprintf(" STRONG TIMES: %i\n",totalstrong);
		myprintf("\n");
	}
	myprintf("* IN ROOT %.2lf\n",root_time);
	myprintf("\n");
	myprintf("LONG DE ZDEPTHS: %i\n",(int)zdepths.size());
	myprintf("TOTAL BASIS: %i\n",totalbasis);
	myprintf("\n");
	return;
}

double tabs(double a) {
	if (a <= 0)
		return -a;
	if (a >= 0)
		return a;
}

int PrintIncInfo(int (*Cij)(int ii, int jj))
{
	double totalobj = 0;
	if(incumbentis)
	{


		for(int i=0;i<nodes;i++)
		{
			totalobj+=Cij(incumbent_x[i],i);
		}
		if(tabs(totalobj-incumbent_value)<tolerance) myprintf("OBJECTIVE VALUE CHECKED: %lf\n",totalobj);
		else {
			myprintf("PROBLEM WITH OBJECTIVE VALUE: CHECKED IS %lf AND INCUMBENT IS %lf\n",totalobj,incumbent_value);
			totalobj = - 1;
		}
		for(int i=0;i<p;i++)
		{
			myprintf("y: %i\n",incumbent_y[i]+1);
		}
		myprintf("\n");
		for(int i=0;i<nodes;i++)
		{
			myprintf("x: %i -> %i\n",i+1,incumbent_x[i]+1);
		}
	}
		return totalobj;
}

void FreeMemory()
{
	for(int i=0;i<nodes;i++)
	{
		Dv[i].clear();
		for(unsigned int j=0;j<S[i].size();j++)
		{
			S[i][j].clear();
		}
		S[i].clear();
	}
	if(totalbasis>0) system("rm *.bas");
	for(int i=0;i<zdepths.size();i++) {
		zdepths[i].clear();	
	}
	zdepths.clear();
	if (RWiter==0) {
		if (G != NULL) 
		{
			 delete []G;
			G = NULL;
		}
	}

//	if(data_format==1)
//	{
//		delete []x;
//		delete []y;
//	}
//	else
//	{
//		for(int i=0;i<nodes;i++)
//		{
//			cost[i].clear(); 
//		}
//		cost.clear();
//	}

//	for(int i=0;i<nodes;i++)
//	{
//		dist[i].clear(); 
//	}
//	dist.clear();

	node_id.clear();
	p_values.clear();
//	b_j.clear();

	Dv.clear();
	S.clear();
	totaldeltar.clear();
	totaldeltal.clear();
	nodesr.clear();
	nodesl.clear();
	initpsr.clear();
	initpsl.clear();
	//incumbent_y.clear();
	//incumbent_x.clear();
	ind_sdz.clear();
	if (G != NULL) {
		delete G;
		G = NULL;
	}
	return;
}

void CompleteProcesses(int (*Cij)(int ii, int jj))
{
	myprintf("ACTIVE NODES: %i\n",active_nodes);
	myprintf("TOTAL NODES: %i\n",total_nodes);
	myprintf("\n");
	if (totalbasis>0) DeleteTree();
	PrintSolInfo();
	PrintIncInfo(Cij);
	if(save_solution_info) fclose(solution_file);
	FreeMemory();
	return;
}
