#include <ilcplex/ilocplex.h>             
#include <ctime>   
ILOSTLBEGIN  
 
#include "global.h"  
#include "auxiliar_functions.h" 
#include "model_functions.h" 
#include "root_node.h"
#include "general_node.h"
#include <sys/time.h>   

/* This version is only valid if costs are integer values */    
   


void zebra_init_global_variables() {
	sdz = 0;
	totalbasis = 0;
	reliab = 0;
	lambda = 0;
	lambda0 = 0;
	lambdamin = 0;
	lambdamax = 0;
	active_nodes = 0;
	total_nodes = 0;
	bestLB = -IloInfinity;
	LB_root = -IloInfinity;
  	incumbent_value = IloInfinity; 
	initial_upper_bound = IloInfinity;
	totalstrong = 0;
}

void store_cij_to_file(char * filename, int nn, int pp, int (*Cij)(int ii, int jj)) {
	FILE * pFile;

	pFile = fopen (filename,"w");
	fprintf(pFile,"%i\n", nn); 
	fprintf(pFile,"%i\n", pp); 

	for(int i=0;i<nn-1;i++)
	{
		for(int j=i+1;j<nn;j++)
		{
			fprintf(pFile,"%i ", Cij(i,j)); 
		}
		fprintf(pFile,"\n");
	}	
	fclose (pFile);

}

void delete_file(char * filename) {
	char command[500];

	sprintf(command,"rm",filename);
	system(command);
} 

int run_zebra(int pp, int nn, int hh, int (*Cij)(int ii, int jj)) {
	// Parameters description:
	// p - number of facilities to be located
	// n - total number of nodes
	// h - how many times Resende-Werneck heuristic should be used
 	// Cij()  - function returning cost associated to the assigment of customer j to the location i
	// incubement_x - resulting assigment of customers
        // incubement_y - list of located facilities

	// set parameters
	zebra_init_global_variables();

	p  	= pp;
	nodes 	= nn;
	RWiter  = hh;

	int objval = 0;
	time_t nowtime;
	time_t aux_time;
	time(&nowtime);

							////////////////////////  	
	// Resende-Werneck heuristic (popstar)
	char orden[500];
	char rwfile[500];
	
	if (RWiter > 0) 
	{	
		char temp_file_name[50] = "temp_cij.dimacs";
		store_cij_to_file(temp_file_name, nodes, p, Cij);
		for(int h=0;h<RWiter;h++) 	
		{	
			strcpy(rwfile,temp_file_name);
			strcat(rwfile,"rw");
			std::string s;
			std::stringstream out;    
			out << h;
			s = out.str();
			strcat(rwfile,s.c_str());
			sprintf(orden,"./mypop %s -p %i -output %s",temp_file_name,p,rwfile);   
			aux_time = clock();
			system(orden);
			Use_RW_info(rwfile);
			heur_time += (double)(clock()-aux_time)/CLOCKS_PER_SEC;
		}
 		delete_file(temp_file_name);
	}
	/* Generate initial data structures */  
	start_time = clock();
	try{
		if (p>nodes)
		{
			myprintf("ERROR: p must be smaller than the number of facilities %i\n",nodes);
			return 0;
		}
		aux_time = clock();
		if(RWiter==0) {
 			Generate_D_sdznoh(Cij);
		} else {
			Generate_D_sdzh(Cij);
		}
		//printf("Generate D time: %lf\n",(double)(clock()-aux_time)/CLOCKS_PER_SEC);
		Start_S();
	}
	catch(const bad_alloc) 
	{
		cout << "ERROR: out of memory while generating D and S." << endl;
		return 0;
	}
    	create_data_time =(double) (clock() - start_time)/CLOCKS_PER_SEC;  
 
	time(&nowtime);
	myprintf("INICIO ROOT %s\n",ctime(&nowtime));
	
	/* Initialize bounds */
	/* Minimization problem */ 
	initial_upper_bound = incumbent_value;
	myprintf("IUB: %lf\n",initial_upper_bound); // Initial Upper Bound  
	/* ROOT NODE */
	active_nodes = 0;
	double rtime=clock();
	Solve_Root_Node(Cij);
	time(&nowtime);
	myprintf("OUT OF ROOT %s\n",ctime(&nowtime));
	root_time = (double)(clock()-rtime)/CLOCKS_PER_SEC;
	total_nodes = 1;
	
	//the_first_active_node->the_info->basis = 0;

	/* Solve the tree */
	double running_time = double(clock()-start_time)/CLOCKS_PER_SEC; 
	while ( (active_nodes>0) && (running_time<time_limit) )
	{
		/* Point the father node */
		this_active_node = the_first_active_node->the_info; 
		if (bestLB>incumbent_value-tolerance)
		{
			/* Optimal reached.
			 Delete the remaining tree */
			while(the_first_active_node->next!=NULL)
			{
				a_n *aux_an;
				aux_an = the_first_active_node;
				the_first_active_node = the_first_active_node->next;
				the_first_active_node->previous = NULL;
				aux_an->next = NULL;
				this_active_node = aux_an->the_info;
				aux_an->the_info = NULL;
				Prune_Node(this_active_node);
				delete aux_an;
			}
			this_active_node = the_first_active_node->the_info;
			the_first_active_node->the_info = NULL;
			Prune_Node(this_active_node);
			this_active_node = NULL;
			delete the_first_active_node;
			the_first_active_node = NULL;
			active_nodes = 0;
			//printf("Tree has been deleted\n");
			break; 
		}
		else
		{
			//printf("Go left\n");
			/* Solve left son node */
			++total_nodes;
			/* Create left son */
			this_active_node->left_son = Create_n_i();
			this_active_node->left_son->parent_node = this_active_node;  
			/* We move to the left son */
			this_active_node = this_active_node->left_son;
			this_active_node->branch_value = 0;
			this_active_node->where_zdepths = this_active_node->parent_node->where_zdepths;
			this_active_node->left_son = NULL;
			this_active_node->right_son = NULL;
			Solve_General_Node(Cij);


			//printf("Go right\n");
			/* Solve right son node */
			++total_nodes;
			/* Point the father node */
			this_active_node 				= the_first_active_node->the_info;
			/* Create right son */
			this_active_node->right_son 			= Create_n_i();
			this_active_node->right_son->parent_node 	= this_active_node;
			/* We move to the right son */
			this_active_node 				= this_active_node->right_son;
			this_active_node->branch_value 			= 1;
			this_active_node->where_zdepths 		= this_active_node->parent_node->where_zdepths;  
			this_active_node->left_son 			= NULL;
			this_active_node->right_son 			= NULL;
			Solve_General_Node(Cij);
		}	
		--active_nodes;
		
		/* Remove parent node from the list*/
		/* Since the_first_active_node is the most promising (smallest lower bound),
		 its two children cannot have a more promosing value */
		if (active_nodes>0)
		{
			a_n *aux_an;
			aux_an = the_first_active_node;
			the_first_active_node = the_first_active_node->next;   
			the_first_active_node->previous = NULL;
			aux_an->next 		= NULL;
			aux_an->previous 	= NULL;
			aux_an->the_info	= NULL;
			delete aux_an;
			aux_an = NULL;
			if(bestLB<incumbent_value-tolerance)
			{
				bestLB = ceil(the_first_active_node->the_info->lower_bound);  
			}
		}
		else
		{
			if ( the_first_active_node != NULL){
				this_active_node = the_first_active_node->the_info;
				the_first_active_node->the_info = NULL;
				//Prune_Node(this_active_node);    // it causes crases due to the fact that Prune_Node has already deallocated the root node(deactivated by Lubos 1.2.2013)
				this_active_node = NULL;	
				delete the_first_active_node;
				the_first_active_node = NULL;
			}
		}
		running_time = (double)(clock()- start_time)/CLOCKS_PER_SEC;
	}
	time(&nowtime);
	myprintf("Total running time: %.0lf\n",running_time); 
	if(data_format==2)
	{
		running_time+= external_generation_time; 
		myprintf("\n");
		myprintf("External data generation time: %.0lf\n",external_generation_time);  
		myprintf("Full total running time: %.0lf\n",running_time);
	}
	if(RW)
	{
		myprintf("\n");
		myprintf("Time in heuristics: %.0lf\n",heur_time);
		myprintf("Total running time including heuristic time: %.0lf\n",running_time+heur_time);  
		myprintf("RW was applied %i time(s).\n",RWiter);
		if(RWiter>1)
		{
			double avhtime = heur_time/RWiter;
			myprintf("Average time in heuristics: %.0lf\n",avhtime);
			myprintf("Average total running time including heuristic time: %.0lf\n",running_time+avhtime); 
		}
	}
	myprintf("\n");
	myprintf("SDZ: %i\n",sdz);
	if(total_nodes>1) myprintf("REL: %i  LAMBDA0 %i\n",reliab,lambda0);
	myprintf("IUB: %lf\n",initial_upper_bound);
	myprintf("\n");
	if (running_time<time_limit) myprintf("STATUS: OPTIMAL VALUE = %lf\n",incumbent_value);
	else
	{
		myprintf("STATUS: TIME LIMIT REACHED WITH %i ACTIVE NODES.\n",active_nodes); 
		myprintf("\n");
		Show_Partial_Solution();
		DeleteTree();
	}
	PrintSolInfo();
	objval = PrintIncInfo(Cij);
	if(save_solution_info) fclose(solution_file);
	FreeMemory();
	DeleteTree();
	return objval;
}

int test_Cij(int i, int j)
{
	if ((i == 0) and (j == 0)) return 0;
	if ((i == 1) and (j == 1)) return 0;
	if ((i == 2) and (j == 2)) return 0;
	if ((i == 3) and (j == 3)) return 0;

	if ((i == 0) and (j == 1)) return 2;
	if ((i == 0) and (j == 2)) return 3;
	if ((i == 0) and (j == 3)) return 1;

	if ((i == 1) and (j == 0)) return 2;
	if ((i == 1) and (j == 2)) return 4;
	if ((i == 1) and (j == 3)) return 3;

	if ((i == 2) and (j == 0)) return 3;
	if ((i == 2) and (j == 1)) return 4;
	if ((i == 2) and (j == 3)) return 1;

	if ((i == 2) and (j == 1)) return 1;
	if ((i == 2) and (j == 2)) return 3;
	if ((i == 2) and (j == 3)) return 1;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int SYSTEM_PROBLEM(int i, int j)
{
	int value = dist[i][j];

	if ((i != j)&&(value <= 0)) {
		printf("Distance value out of scope i = %i, j = %i, value = %i \n",i,j,value);
		exit(0);
	} 
	if (i == j) return 0;
	return value;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int calculate_SYSTEM_OPTIMA()
{
	for(int p_p = 0; p_p < num_p_values;p_p++)
	{
		if (verbose_jlm)
		{
			printf("Launching calculation for p = %i \n",p_values[p_p]);
		}
		incumbent_y.resize(p_values[p_p]);  
		incumbent_x.resize(nodes);
		int obj = run_zebra(p_values[p_p],nodes,RWiter,SYSTEM_PROBLEM);
		incumbent_y.clear();
		incumbent_x.clear();
	}
	return 1;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int calculate_SYSTEM_OPTIMUM(int p)
{
	if (verbose_jlm)
	{
		printf("Launching calculation for p = %i \n",p);
	}
	incumbent_y.resize(p);  
	incumbent_x.resize(nodes);
	int obj = run_zebra(p,nodes,RWiter,SYSTEM_PROBLEM);
	incumbent_y.clear();
	incumbent_x.clear();
	return obj;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int determine_Dk_values(int Max_value)
{
	vector<int> occurence;
	occurence.resize(Max_value + 1);
	// count how many times each value smaller or equal than Max_value appears int the dist matrix	
	for(int i = 0 ; i < nodes; i++)
	{
		for(int j = 0; j < nodes; j++)
		{
			if (dist[i][j] <= Max_value) 
			{
				occurence[dist[i][j]] += 1; 
			}
		}
	}

	int num_Dk_values = 0;
	// create Dk array
	for(int i=0; i< Max_value + 1; i++) {
		if (occurence[i] > 0) {
			Dk[num_Dk_values] = i;
			num_Dk_values++;
		}	
	}

	occurence.clear();
	return num_Dk_values-1;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int dej_rijk(int Dt) 
{
	printf("********************************************************************\n");
	printf("********************************************************************\n");
	printf("********************************************************************\n");
	printf("Dt = %i\n", Dt);
	int Num_sin_cust = 0;

	// determine number of single customers
	for(int i=0; i < nodes; i++) 
	{
		if (Akt[i] == 0) Num_sin_cust += b_j[i];
	}
	if (verbose_jlm)
	{	
		printf("Num_sin_cust = %i\n", Num_sin_cust);
	}

	// determine rijk values
	for(int j = 0; j < nodes; j++)
	{	
		if (Akt[j] == 0) 
		{ 	// active customer
			for(int i=0; i < nodes; i++)
			{
				if (dist[i][j] < Dt)
					rijk[i][j] =  0 + ZEBRA_SHIFT;
				if (dist[i][j] == Dt) 
				{	
					rijk[i][j] = JLM_EPSILON*b_j[j] + ZEBRA_SHIFT;	
					if (rijk[i][j] <= 0) 
					{
						printf("i = %i, j = %i, b_j[%i] = %i",i,j,j,b_j[j]);
						exit(0);
					}
				}
				if (dist[i][j] > Dt)
				{				
					//rijk[i][j] = (dist[i][j] - Dt + JLM_EPSILON*ceil((Num_sin_cust+0.0)/(b_j[j]+0.0)))*b_j[j] + ZEBRA_SHIFT; 
					rijk[i][j] = JLM_EPSILON*(1+ Num_sin_cust) + ZEBRA_SHIFT; 
					if (rijk[i][j] <= 0) 
					{
						printf("i = %i, j = %i, dist[%i][%i] = %i, Dt = %i, b_j[%i] = %i\n",i,j,i,j,dist[i][j],Dt,j,b_j[j]);
						exit(0);
					}
				}
			}
		} else
		{	// non active customer 
			for(int i = 0; i < nodes; i++)
			{
				if (dist[i][j] <= Akt[j])	
					rijk[i][j] = 0 + ZEBRA_SHIFT;
				else
				{
					//rijk[i][j] = (max(JLM_EPSILON, dist[i][j] - Akt[j])	+ JLM_EPSILON*ceil((Num_sin_cust+0.0)/(b_j[j]+0.0))   )*b_j[j] + ZEBRA_SHIFT;
					rijk[i][j] = JLM_EPSILON*(1+ Num_sin_cust) + ZEBRA_SHIFT;
					if (rijk[i][j] <= 0) 
					{
						printf("i = %i, j = %i, dist[%i][%i] = %i, Akt[%i] = %i\n",i,j,i,j,dist[i][j],j,Akt[j]);
						exit(0);
					}
				}
			}
		}	
	}
	return Num_sin_cust;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int zebra_rijk(int i, int j)
{
	int value = rijk[i][j];

	if ((i < 0)||(j < 0)) {
		printf("Nagative values i = %i, j = %i\n");
		exit(0);
	}

	if ((i != j)&&(value <= 0)) {
		printf("rijk value out of valid range i = %i, j = %i, value = %i\n",i,j,value);
		exit(0);
	} 
	if (i == j) return 0;
	return value;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int recalculate_objval()
{ // recalculate objval (the main purpose of this is to clean from objval ZEBRA_SHIFT VALUES)
	int objval = 0;
	for(int j = 0; j < nodes; j++)
	{
		if (incumbent_x[j] != j)
			objval += rijk[incumbent_x[j]][j] - ZEBRA_SHIFT;
	}
	return objval;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int calculate_LEXMIN_JLM_ALGORITHM()
{
	// alocate memory

	timeval t1, t2;
    	double elapsedTime3;

	rijk.resize(nodes);
	Akt.resize(nodes);
	for(int i=0; i < nodes; i++) 
	{	
		rijk[i].resize(nodes);
	}

	output_sys_val = fopen("sys_val.txt","w");
	output_sys_ass = fopen("sys_ass.txt","w");
	output_sys_loc = fopen("sys_loc.txt","w");

	for(int p_p = 0; p_p < num_p_values;p_p++)
	{	

		incumbent_y.resize(p_values[p_p]);  
		incumbent_x.resize(nodes);

		char file_name[50];
		sprintf(file_name,"out_%i.txt",p_values[p_p]);
		output1 = fopen(file_name,"w");

		sprintf(file_name,"loc_%i.txt",p_values[p_p]);
		output_loc = fopen(file_name,"w");

		sprintf(file_name,"ass_%i.txt",p_values[p_p]);
		output_ass = fopen(file_name,"w");

		sprintf(file_name,"val_%i.txt",p_values[p_p]);
		output_val = fopen(file_name,"w");

		sprintf(file_name,"global_%i.txt",p_values[p_p]);
		output_global = fopen(file_name,"w");

		gettimeofday(&t1, NULL);


		fprintf(output1,"Dk; processing time [ms]; obj_val;fixed customers;\n");

		for(int i=0; i < nodes; i++) 
		{		
			Akt[i] = 0;
		}
		if (verbose_jlm)
		{
			printf("Launching calculation for p = %i \n",p_values[p_p]);
			printf("Calculating system optimum ... \n");
		}
		int obj = run_zebra(p_values[p_p],nodes,RWiter,SYSTEM_PROBLEM);

		// store locations
		for (int j = 0; j < p_values[p_p]; j++) 
		{
			fprintf(output_sys_loc, "%i\n",incumbent_y[j]+1);
		}	
		// store assignments	
		for (int j = 0; j < nodes; j++) 
		{
			fprintf(output_sys_ass, "%i\n",incumbent_x[j]+1);
		}	

		// store assigned values
		for (int j = 0; j < nodes; j++) 
		{
			fprintf(output_sys_val, "%i\n",  dist[incumbent_x[j]][j] );
		}
		fclose(output_sys_loc);
		fclose(output_sys_ass);
		fclose(output_sys_val);
		if (verbose_jlm)
		{
			printf("System optimum calculated, objval = %i\n",obj);
		}
		int AktC = nodes;

		// find maximum assignment
		int Max_Value = -1;
		for(int i=0; i < nodes; i++)
		{	
			if (Max_Value < dist[incumbent_x[i]][i])
			{
				Max_Value = dist[incumbent_x[i]][i];		
			}
		}
		if (verbose_jlm)
		{
			printf("Max_Value = %i\n",Max_Value);
		}
		incumbent_y.clear();
		incumbent_x.clear();
		// determine Dk values
		Dk.resize(Max_Value+1);
		int Dk_Pos  =  determine_Dk_values(Max_Value);

		while((AktC - p_values[p_p] > 0)&&(Dk_Pos >= 0))   // process dk values
		{
			printf("AktC = %i\n",AktC);
			if (verbose_jlm)
			{
				printf("Processing Dk = %i, \n",Dk[Dk_Pos]);
				printf("Dk_Pos = %i \n", Dk_Pos);
				//for(int i = 0; i< Max_Value; i++)
				//{
					//printf("Dk[%i] = %i \n", i, Dk[i]); 
				//}
			}

			
			int Act_cust = dej_rijk(Dk[Dk_Pos]);
			
			timeval t1_dt, t2_dt;
    			double elapsedTime_dt;


			gettimeofday(&t1_dt, NULL);
			int obj = run_zebra(p_values[p_p],nodes,RWiter,zebra_rijk);
			gettimeofday(&t2_dt, NULL);
    	
			elapsedTime_dt  = (t2_dt.tv_sec  - t1_dt.tv_sec) * 1000.0;      // sec to ms
    			elapsedTime_dt += (t2_dt.tv_usec - t1_dt.tv_usec) / 1000.0;     // us to ms


			int f   = recalculate_objval();

	

			if (verbose_jlm) 
			{
				printf("objval = %i\n",f);
			}	
			if (f > JLM_EPSILON*Act_cust) 
			{
				printf("Objective function value is out of the scope. Such situation in JLM should not appear.\n");
				exit(0);
			}

			int num_fixed = 0;
			if ((f >= JLM_EPSILON)&&(f <= JLM_EPSILON*Act_cust)) 
			{
				for(int j = 0; j < nodes; j++)
				{
					if (Akt[j] == 0) 
					{
						if (rijk[incumbent_x[j]][j] - ZEBRA_SHIFT == JLM_EPSILON*b_j[j]) 	
						{
							Akt[j] = Dk[Dk_Pos];
							AktC--;	
							num_fixed++;
							if (verbose_jlm)
							{
								printf("FIXING CUSTOMER: %i\n",j);
							}
						}
					}
				}
			}

			fprintf(output1,"%i;%f;%i,%i;%i\n",Dk[Dk_Pos],elapsedTime_dt,f,num_fixed,Dk_Pos);	

			if (AktC - p_values[p_p] > 0)
			{ 	// reset Dk value
				Dk_Pos--;
				// check is some following Dk values can be skipped from the evaluation
				int Fp = 0;
				if (Dk_Pos > 0)
				{
					do {				
						Fp = 0;
						for (int j = 0; j < nodes; j++) 
						{
							if (dist[incumbent_x[j]][j] == Dk[Dk_Pos])
							{
								Fp += b_j[j];
							}	
							//printf("%i %i\n",j,dist[incumbent_x[j]][j]);
						}
						if (Fp <= 0.1)
						{
							fprintf(output1,"Value %i jumped over. Fp = %i\n",Dk[Dk_Pos], Fp);
							Dk_Pos--;
						}
					}					
					while((Fp < 0.1)&&( Dk_Pos > 0));
				}
			}

		}
		gettimeofday(&t2, NULL);
		// store locations
		for (int j = 0; j < p_values[p_p]; j++) 
		{
			fprintf(output_loc, "%i\n",incumbent_y[j]+1);

		}	
		// store assignments	
		for (int j = 0; j < nodes; j++) 
		{
			fprintf(output_ass, "%i\n",incumbent_x[j]+1);
		}	

		// store assigned values
		for (int j = 0; j < nodes; j++) 
		{
			fprintf(output_val, "%i\n",  dist[incumbent_x[j]][j] );
		}	
    		elapsedTime3  = (t2.tv_sec  - t1.tv_sec) * 1000.0;      // sec to ms
    		elapsedTime3 += (t2.tv_usec - t1.tv_usec) / 1000.0;     // us to ms
		fprintf(output_global, "Elapsed time: %f ms",elapsedTime3);
		fclose(output1);
		fclose(output_loc);
		fclose(output_ass);
		fclose(output_val);
		fclose(output_global);
		incumbent_y.clear();
		incumbent_x.clear();
	}
	// release memory
	Dk.clear();
	Akt.clear();

	for(int i = 0; i < nodes; i++)
	{
		rijk[i].clear();
	}
	rijk.clear();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{	
	if(argc!=5) 
	{
		printf("ERROR: wrong command line.\n");
		printf("The structure is: z_reduced_main Directory_network Directory_problem [-h h]\n");     
		printf("\n");
		printf("Directory_network  = directory containing the definition of the network\n");
		printf("Directory_problem  = diretory with the definition of the problem\n");
		printf("h = apply h times Popstar heuristic (Resende-Werneck) before starting.\n");   
		return 0;
	}
	else
	{
		/////////////////////////////// READ PARAMETERS ////////////////////////////////
		Directory_Network = argv[1];
		Directory_Problem = argv[2];
		int arg = 3;

		if(strcmp(arg[argv],"-h")==0)
		{
			RW = true;
			++arg;
			if(arg==argc) return false;
			RWiter = atoi(argv[arg]);
			++arg;
		}
		/////////////////////////////// READ FILES WITH THE PROBLEM //////////////////////////////// 
		char temp_file_name[500];
		sprintf(temp_file_name,"%s/%s",Directory_Network,"matVzdialenosti.txt");
		if (Read_all_distances(temp_file_name) == 0)
		{
			if (verbose_jlm)
			{
				printf("Reading file with all distances failed\n");
			}
			exit(0);
		}
		sprintf(temp_file_name,"%s/%s",Directory_Problem,"maxP.txt");
		if (Read_p_values(temp_file_name) == 0)
		{
			if (verbose_jlm)
			{
				printf("Reading file with p values failed\n");
			}
			exit(0);
		}
		sprintf(temp_file_name,"%s/%s",Directory_Problem,"skladyId.txt"); 
		if (Read_node_id(temp_file_name) == 0)
		{
			if (verbose_jlm)
			{
				printf("Reading file with node ids failed\n");
			}
			exit(0);
		}
		sprintf(temp_file_name,"%s/%s",Directory_Network,"zakazniciPocObyv.txt");
		if (Read_poc_obyv(temp_file_name) == 0)
		{
			if (verbose_jlm)
			{
				printf("Reading file with poc_obyv failed\n");
			}
			exit(0);
		}
		if (verbose_jlm)
		{
			printf("All data sucesfully read from files ...\n");
		}	
		/////////////////////////////// EXTRACT INPUTS FOR THE PROBLEM //////////////////////////////// 
		b_j.resize(nodes);
		for(int i =0; i < nodes;i++)
		{
			b_j[i] = poc_obyv[i];
			//b_j[i] = poc_obyv[node_id[i]-1];

			if (b_j[i] <= 0) 
			{
				printf("b_j[%i] = %i is out of valid range\n",i,b_j[i]);
				exit(0);
			}   
		} 
		// release poc_obyv
		poc_obyv.clear(); 
		//if (verbose_jlm)
		//{
			//for(int i =0; i < nodes;i++)
			//{
				//printf("b_j[%i] = %i\n", i, b_j[i]);
			//}
		//}
		dist.resize(nodes);
		for(int i = 0; i < nodes;i++)
		{
			dist[i].resize(nodes);
	
			for(int j = 0; j < nodes; j++) 
			{
				dist[i][j] = cost[node_id[i]-1][node_id[j]-1] + 1;
			}
			dist[i][i] = 0; 
		}
		// release total cost matrix
		for(int i=0;i<all_nodes;i++)
		{
			cost[i].clear(); 
		}
		cost.clear();
		if (verbose_jlm)
		{
			printf("All data structures generated ...\n");
		}
	}



	///////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////
	//calculate_SYSTEM_OPTIMA();
	calculate_LEXMIN_JLM_ALGORITHM();
	///////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////

	for(int i=0;i<nodes;i++)
	{
		dist[i].clear(); 
	}
	dist.clear();
	b_j.clear();
	return 0;
}
